/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <Unity/Application/application_manager.h>

#include <Unity/Application/applicationcontroller.h>
#include <Unity/Application/taskcontroller.h>

#include <core/posix/linux/proc/process/oom_score_adj.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>

#include "mock_application_controller.h"
#include "mock_desktop_file_reader.h"
#include "mock_oom_controller.h"
#include "mock_process_controller.h"

TEST(ApplicationManager, SuspendingAndResumingARunningApplicationResultsInOomScoreAdjustment)
{
    using namespace ::testing;

    const QString appId("com.canonical.does.not.exist");

    NiceMock<testing::MockOomController> oomController;
    QSharedPointer<ProcessController::OomController> oomControllerPtr(
                &oomController,
                [](ProcessController::OomController*){});

    NiceMock<testing::MockProcessController> processController(oomControllerPtr);
    QSharedPointer<ProcessController> processControllerPtr(
                &processController,
                [](ProcessController*){});

    NiceMock<testing::MockApplicationController> appController;
    QSharedPointer<ApplicationController> appControllerPtr(
                &appController,
                [](ApplicationController*){});

    QSharedPointer<TaskController> taskController(
                new TaskController(
                    nullptr,
                    appControllerPtr,
                    processControllerPtr
                    ));

    NiceMock<MockDesktopFileReaderFactory> desktopFileReaderFactory;
    QSharedPointer<DesktopFileReader::Factory> desktopFileReaderFactoryPtr(
                &desktopFileReaderFactory,
                [](DesktopFileReader::Factory*){});

    ApplicationManager applicationManager(
                taskController,
                desktopFileReaderFactoryPtr);

    EXPECT_CALL(appController, startApplicationWithAppIdAndArgs(_, _)).Times(1);

    EXPECT_CALL(desktopFileReaderFactory, createInstanceForAppId(appId)).Times(1);

    EXPECT_CALL(processController, sigStopProcessGroupForPid(_)).Times(1);
    EXPECT_CALL(processController, sigContinueProcessGroupForPid(_)).Times(1);
    EXPECT_CALL(oomController, ensureProcessUnlikelyToBeKilled(_)).Times(1);
    EXPECT_CALL(oomController, ensureProcessLikelyToBeKilled(_)).Times(1);

    auto application = applicationManager.startApplication(
                appId,
                ApplicationManager::NoFlag,
                QStringList());

    application->suspend();
    application->resume();
}

// Currently disabled as we need to make sure that we have a corresponding mir session, too.
TEST(ApplicationManager, DISABLED_FocusingRunningApplicationResultsInOomScoreAdjustment)
{
    using namespace ::testing;

    const QString appId("com.canonical.does.not.exist");

    NiceMock<testing::MockOomController> oomController;
    QSharedPointer<ProcessController::OomController> oomControllerPtr(
                &oomController,
                [](ProcessController::OomController*){});

    NiceMock<testing::MockProcessController> processController(oomControllerPtr);
    QSharedPointer<ProcessController> processControllerPtr(
                &processController,
                [](ProcessController*){});

    NiceMock<testing::MockApplicationController> appController;
    QSharedPointer<ApplicationController> appControllerPtr(
                &appController,
                [](ApplicationController*){});

    QSharedPointer<TaskController> taskController(
                new TaskController(
                    nullptr,
                    appControllerPtr,
                    processControllerPtr
                    ));

    NiceMock<MockDesktopFileReaderFactory> desktopFileReaderFactory;
    QSharedPointer<DesktopFileReader::Factory> desktopFileReaderFactoryPtr(
                &desktopFileReaderFactory,
                [](DesktopFileReader::Factory*){});

    ApplicationManager applicationManager(
                taskController,
                desktopFileReaderFactoryPtr);

    QSet<QString> appIds;

    for (unsigned int i = 0; i < 50; i++)
    {
        QString appIdFormat("%1.does.not.exist");
        auto appId = appIdFormat.arg(i);

        auto application = applicationManager.startApplication(
                    appId,
                    ApplicationManager::NoFlag,
                    QStringList());

        EXPECT_NE(nullptr, application);

        appIds.insert(appId);
        auto it = appController.children.find(appId);
        if (it != appController.children.end())
            EXPECT_CALL(oomController,
                        ensureProcessUnlikelyToBeKilled(it->pid())).Times(1);
    }

    for (auto appId : appIds)
    {
        applicationManager.focusApplication(appId);
    }
}
